# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: kuksa/val/v1/val.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'kuksa/val/v1/val.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from kuksa.val.v1 import types_pb2 as kuksa_dot_val_dot_v1_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16kuksa/val/v1/val.proto\x12\x0ckuksa.val.v1\x1a\x18kuksa/val/v1/types.proto\"c\n\x0c\x45ntryRequest\x12\x0c\n\x04path\x18\x01 \x01(\t\x12 \n\x04view\x18\x02 \x01(\x0e\x32\x12.kuksa.val.v1.View\x12#\n\x06\x66ields\x18\x03 \x03(\x0e\x32\x13.kuksa.val.v1.Field\"9\n\nGetRequest\x12+\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1a.kuksa.val.v1.EntryRequest\"\x89\x01\n\x0bGetResponse\x12(\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x17.kuksa.val.v1.DataEntry\x12,\n\x06\x65rrors\x18\x02 \x03(\x0b\x32\x1c.kuksa.val.v1.DataEntryError\x12\"\n\x05\x65rror\x18\x03 \x01(\x0b\x32\x13.kuksa.val.v1.Error\"Z\n\x0b\x45ntryUpdate\x12&\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x17.kuksa.val.v1.DataEntry\x12#\n\x06\x66ields\x18\x02 \x03(\x0e\x32\x13.kuksa.val.v1.Field\"8\n\nSetRequest\x12*\n\x07updates\x18\x01 \x03(\x0b\x32\x19.kuksa.val.v1.EntryUpdate\"_\n\x0bSetResponse\x12\"\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x13.kuksa.val.v1.Error\x12,\n\x06\x65rrors\x18\x02 \x03(\x0b\x32\x1c.kuksa.val.v1.DataEntryError\"C\n\x15StreamedUpdateRequest\x12*\n\x07updates\x18\x01 \x03(\x0b\x32\x19.kuksa.val.v1.EntryUpdate\"j\n\x16StreamedUpdateResponse\x12\"\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x13.kuksa.val.v1.Error\x12,\n\x06\x65rrors\x18\x02 \x03(\x0b\x32\x1c.kuksa.val.v1.DataEntryError\"e\n\x0eSubscribeEntry\x12\x0c\n\x04path\x18\x01 \x01(\t\x12 \n\x04view\x18\x02 \x01(\x0e\x32\x12.kuksa.val.v1.View\x12#\n\x06\x66ields\x18\x03 \x03(\x0e\x32\x13.kuksa.val.v1.Field\"A\n\x10SubscribeRequest\x12-\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1c.kuksa.val.v1.SubscribeEntry\"?\n\x11SubscribeResponse\x12*\n\x07updates\x18\x01 \x03(\x0b\x32\x19.kuksa.val.v1.EntryUpdate\"\x16\n\x14GetServerInfoRequest\"6\n\x15GetServerInfoResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t2\x88\x03\n\x03VAL\x12:\n\x03Get\x12\x18.kuksa.val.v1.GetRequest\x1a\x19.kuksa.val.v1.GetResponse\x12:\n\x03Set\x12\x18.kuksa.val.v1.SetRequest\x1a\x19.kuksa.val.v1.SetResponse\x12_\n\x0eStreamedUpdate\x12#.kuksa.val.v1.StreamedUpdateRequest\x1a$.kuksa.val.v1.StreamedUpdateResponse(\x01\x30\x01\x12N\n\tSubscribe\x12\x1e.kuksa.val.v1.SubscribeRequest\x1a\x1f.kuksa.val.v1.SubscribeResponse0\x01\x12X\n\rGetServerInfo\x12\".kuksa.val.v1.GetServerInfoRequest\x1a#.kuksa.val.v1.GetServerInfoResponseB\x0eZ\x0ckuksa/val/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kuksa.val.v1.val_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\014kuksa/val/v1'
  _globals['_ENTRYREQUEST']._serialized_start=66
  _globals['_ENTRYREQUEST']._serialized_end=165
  _globals['_GETREQUEST']._serialized_start=167
  _globals['_GETREQUEST']._serialized_end=224
  _globals['_GETRESPONSE']._serialized_start=227
  _globals['_GETRESPONSE']._serialized_end=364
  _globals['_ENTRYUPDATE']._serialized_start=366
  _globals['_ENTRYUPDATE']._serialized_end=456
  _globals['_SETREQUEST']._serialized_start=458
  _globals['_SETREQUEST']._serialized_end=514
  _globals['_SETRESPONSE']._serialized_start=516
  _globals['_SETRESPONSE']._serialized_end=611
  _globals['_STREAMEDUPDATEREQUEST']._serialized_start=613
  _globals['_STREAMEDUPDATEREQUEST']._serialized_end=680
  _globals['_STREAMEDUPDATERESPONSE']._serialized_start=682
  _globals['_STREAMEDUPDATERESPONSE']._serialized_end=788
  _globals['_SUBSCRIBEENTRY']._serialized_start=790
  _globals['_SUBSCRIBEENTRY']._serialized_end=891
  _globals['_SUBSCRIBEREQUEST']._serialized_start=893
  _globals['_SUBSCRIBEREQUEST']._serialized_end=958
  _globals['_SUBSCRIBERESPONSE']._serialized_start=960
  _globals['_SUBSCRIBERESPONSE']._serialized_end=1023
  _globals['_GETSERVERINFOREQUEST']._serialized_start=1025
  _globals['_GETSERVERINFOREQUEST']._serialized_end=1047
  _globals['_GETSERVERINFORESPONSE']._serialized_start=1049
  _globals['_GETSERVERINFORESPONSE']._serialized_end=1103
  _globals['_VAL']._serialized_start=1106
  _globals['_VAL']._serialized_end=1498
# @@protoc_insertion_point(module_scope)
