# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: kuksa/val/v2/types.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'kuksa/val/v2/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18kuksa/val/v2/types.proto\x12\x0ckuksa.val.v2\x1a\x1fgoogle/protobuf/timestamp.proto\"^\n\tDatapoint\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.kuksa.val.v2.Value\"\xaf\x04\n\x05Value\x12\x10\n\x06string\x18\x0b \x01(\tH\x00\x12\x0e\n\x04\x62ool\x18\x0c \x01(\x08H\x00\x12\x0f\n\x05int32\x18\r \x01(\x11H\x00\x12\x0f\n\x05int64\x18\x0e \x01(\x12H\x00\x12\x10\n\x06uint32\x18\x0f \x01(\rH\x00\x12\x10\n\x06uint64\x18\x10 \x01(\x04H\x00\x12\x0f\n\x05\x66loat\x18\x11 \x01(\x02H\x00\x12\x10\n\x06\x64ouble\x18\x12 \x01(\x01H\x00\x12\x31\n\x0cstring_array\x18\x15 \x01(\x0b\x32\x19.kuksa.val.v2.StringArrayH\x00\x12-\n\nbool_array\x18\x16 \x01(\x0b\x32\x17.kuksa.val.v2.BoolArrayH\x00\x12/\n\x0bint32_array\x18\x17 \x01(\x0b\x32\x18.kuksa.val.v2.Int32ArrayH\x00\x12/\n\x0bint64_array\x18\x18 \x01(\x0b\x32\x18.kuksa.val.v2.Int64ArrayH\x00\x12\x31\n\x0cuint32_array\x18\x19 \x01(\x0b\x32\x19.kuksa.val.v2.Uint32ArrayH\x00\x12\x31\n\x0cuint64_array\x18\x1a \x01(\x0b\x32\x19.kuksa.val.v2.Uint64ArrayH\x00\x12/\n\x0b\x66loat_array\x18\x1b \x01(\x0b\x32\x18.kuksa.val.v2.FloatArrayH\x00\x12\x31\n\x0c\x64ouble_array\x18\x1c \x01(\x0b\x32\x19.kuksa.val.v2.DoubleArrayH\x00\x42\r\n\x0btyped_value\"2\n\x08SignalID\x12\x0c\n\x02id\x18\x01 \x01(\x05H\x00\x12\x0e\n\x04path\x18\x02 \x01(\tH\x00\x42\x08\n\x06signal\"?\n\x05\x45rror\x12%\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x17.kuksa.val.v2.ErrorCode\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xb6\x02\n\x08Metadata\x12\x0c\n\x04path\x18\t \x01(\t\x12\n\n\x02id\x18\n \x01(\x05\x12)\n\tdata_type\x18\x0b \x01(\x0e\x32\x16.kuksa.val.v2.DataType\x12+\n\nentry_type\x18\x0c \x01(\x0e\x32\x17.kuksa.val.v2.EntryType\x12\x13\n\x0b\x64\x65scription\x18\r \x01(\t\x12\x0f\n\x07\x63omment\x18\x0e \x01(\t\x12\x13\n\x0b\x64\x65precation\x18\x0f \x01(\t\x12\x0c\n\x04unit\x18\x10 \x01(\t\x12+\n\x0e\x61llowed_values\x18\x11 \x01(\x0b\x32\x13.kuksa.val.v2.Value\x12 \n\x03min\x18\x12 \x01(\x0b\x32\x13.kuksa.val.v2.Value\x12 \n\x03max\x18\x13 \x01(\x0b\x32\x13.kuksa.val.v2.Value\"\x1d\n\x0bStringArray\x12\x0e\n\x06values\x18\x01 \x03(\t\"\x1b\n\tBoolArray\x12\x0e\n\x06values\x18\x01 \x03(\x08\"\x1c\n\nInt32Array\x12\x0e\n\x06values\x18\x01 \x03(\x11\"\x1c\n\nInt64Array\x12\x0e\n\x06values\x18\x01 \x03(\x12\"\x1d\n\x0bUint32Array\x12\x0e\n\x06values\x18\x01 \x03(\r\"\x1d\n\x0bUint64Array\x12\x0e\n\x06values\x18\x01 \x03(\x04\"\x1c\n\nFloatArray\x12\x0e\n\x06values\x18\x01 \x03(\x02\"\x1d\n\x0b\x44oubleArray\x12\x0e\n\x06values\x18\x01 \x03(\x01*\x97\x01\n\tErrorCode\x12\x1a\n\x16\x45RROR_CODE_UNSPECIFIED\x10\x00\x12\x11\n\rERROR_CODE_OK\x10\x01\x12\x1f\n\x1b\x45RROR_CODE_INVALID_ARGUMENT\x10\x02\x12\x18\n\x14\x45RROR_CODE_NOT_FOUND\x10\x03\x12 \n\x1c\x45RROR_CODE_PERMISSION_DENIED\x10\x04*\xa9\x05\n\x08\x44\x61taType\x12\x19\n\x15\x44\x41TA_TYPE_UNSPECIFIED\x10\x00\x12\x14\n\x10\x44\x41TA_TYPE_STRING\x10\x01\x12\x15\n\x11\x44\x41TA_TYPE_BOOLEAN\x10\x02\x12\x12\n\x0e\x44\x41TA_TYPE_INT8\x10\x03\x12\x13\n\x0f\x44\x41TA_TYPE_INT16\x10\x04\x12\x13\n\x0f\x44\x41TA_TYPE_INT32\x10\x05\x12\x13\n\x0f\x44\x41TA_TYPE_INT64\x10\x06\x12\x13\n\x0f\x44\x41TA_TYPE_UINT8\x10\x07\x12\x14\n\x10\x44\x41TA_TYPE_UINT16\x10\x08\x12\x14\n\x10\x44\x41TA_TYPE_UINT32\x10\t\x12\x14\n\x10\x44\x41TA_TYPE_UINT64\x10\n\x12\x13\n\x0f\x44\x41TA_TYPE_FLOAT\x10\x0b\x12\x14\n\x10\x44\x41TA_TYPE_DOUBLE\x10\x0c\x12\x17\n\x13\x44\x41TA_TYPE_TIMESTAMP\x10\r\x12\x1a\n\x16\x44\x41TA_TYPE_STRING_ARRAY\x10\x14\x12\x1b\n\x17\x44\x41TA_TYPE_BOOLEAN_ARRAY\x10\x15\x12\x18\n\x14\x44\x41TA_TYPE_INT8_ARRAY\x10\x16\x12\x19\n\x15\x44\x41TA_TYPE_INT16_ARRAY\x10\x17\x12\x19\n\x15\x44\x41TA_TYPE_INT32_ARRAY\x10\x18\x12\x19\n\x15\x44\x41TA_TYPE_INT64_ARRAY\x10\x19\x12\x19\n\x15\x44\x41TA_TYPE_UINT8_ARRAY\x10\x1a\x12\x1a\n\x16\x44\x41TA_TYPE_UINT16_ARRAY\x10\x1b\x12\x1a\n\x16\x44\x41TA_TYPE_UINT32_ARRAY\x10\x1c\x12\x1a\n\x16\x44\x41TA_TYPE_UINT64_ARRAY\x10\x1d\x12\x19\n\x15\x44\x41TA_TYPE_FLOAT_ARRAY\x10\x1e\x12\x1a\n\x16\x44\x41TA_TYPE_DOUBLE_ARRAY\x10\x1f\x12\x1d\n\x19\x44\x41TA_TYPE_TIMESTAMP_ARRAY\x10 *q\n\tEntryType\x12\x1a\n\x16\x45NTRY_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14\x45NTRY_TYPE_ATTRIBUTE\x10\x01\x12\x15\n\x11\x45NTRY_TYPE_SENSOR\x10\x02\x12\x17\n\x13\x45NTRY_TYPE_ACTUATOR\x10\x03\x42\x0eZ\x0ckuksa/val/v2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'kuksa.val.v2.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\014kuksa/val/v2'
  _globals['_ERRORCODE']._serialized_start=1407
  _globals['_ERRORCODE']._serialized_end=1558
  _globals['_DATATYPE']._serialized_start=1561
  _globals['_DATATYPE']._serialized_end=2242
  _globals['_ENTRYTYPE']._serialized_start=2244
  _globals['_ENTRYTYPE']._serialized_end=2357
  _globals['_DATAPOINT']._serialized_start=75
  _globals['_DATAPOINT']._serialized_end=169
  _globals['_VALUE']._serialized_start=172
  _globals['_VALUE']._serialized_end=731
  _globals['_SIGNALID']._serialized_start=733
  _globals['_SIGNALID']._serialized_end=783
  _globals['_ERROR']._serialized_start=785
  _globals['_ERROR']._serialized_end=848
  _globals['_METADATA']._serialized_start=851
  _globals['_METADATA']._serialized_end=1161
  _globals['_STRINGARRAY']._serialized_start=1163
  _globals['_STRINGARRAY']._serialized_end=1192
  _globals['_BOOLARRAY']._serialized_start=1194
  _globals['_BOOLARRAY']._serialized_end=1221
  _globals['_INT32ARRAY']._serialized_start=1223
  _globals['_INT32ARRAY']._serialized_end=1251
  _globals['_INT64ARRAY']._serialized_start=1253
  _globals['_INT64ARRAY']._serialized_end=1281
  _globals['_UINT32ARRAY']._serialized_start=1283
  _globals['_UINT32ARRAY']._serialized_end=1312
  _globals['_UINT64ARRAY']._serialized_start=1314
  _globals['_UINT64ARRAY']._serialized_end=1343
  _globals['_FLOATARRAY']._serialized_start=1345
  _globals['_FLOATARRAY']._serialized_end=1373
  _globals['_DOUBLEARRAY']._serialized_start=1375
  _globals['_DOUBLEARRAY']._serialized_end=1404
# @@protoc_insertion_point(module_scope)
