# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: sdv/databroker/v1/types.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'sdv/databroker/v1/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dsdv/databroker/v1/types.proto\x12\x11sdv.databroker.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"\x1d\n\x0bStringArray\x12\x0e\n\x06values\x18\x01 \x03(\t\"\x1b\n\tBoolArray\x12\x0e\n\x06values\x18\x01 \x03(\x08\"\x1c\n\nInt32Array\x12\x0e\n\x06values\x18\x01 \x03(\x11\"\x1c\n\nInt64Array\x12\x0e\n\x06values\x18\x01 \x03(\x12\"\x1d\n\x0bUint32Array\x12\x0e\n\x06values\x18\x01 \x03(\r\"\x1d\n\x0bUint64Array\x12\x0e\n\x06values\x18\x01 \x03(\x04\"\x1c\n\nFloatArray\x12\x0e\n\x06values\x18\x01 \x03(\x02\"\x1d\n\x0b\x44oubleArray\x12\x0e\n\x06values\x18\x01 \x03(\x01\"\xe2\x06\n\tDatapoint\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12=\n\rfailure_value\x18\n \x01(\x0e\x32$.sdv.databroker.v1.Datapoint.FailureH\x00\x12\x16\n\x0cstring_value\x18\x0b \x01(\tH\x00\x12\x14\n\nbool_value\x18\x0c \x01(\x08H\x00\x12\x15\n\x0bint32_value\x18\r \x01(\x11H\x00\x12\x15\n\x0bint64_value\x18\x0e \x01(\x12H\x00\x12\x16\n\x0cuint32_value\x18\x0f \x01(\rH\x00\x12\x16\n\x0cuint64_value\x18\x10 \x01(\x04H\x00\x12\x15\n\x0b\x66loat_value\x18\x11 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x12 \x01(\x01H\x00\x12\x36\n\x0cstring_array\x18\x15 \x01(\x0b\x32\x1e.sdv.databroker.v1.StringArrayH\x00\x12\x32\n\nbool_array\x18\x16 \x01(\x0b\x32\x1c.sdv.databroker.v1.BoolArrayH\x00\x12\x34\n\x0bint32_array\x18\x17 \x01(\x0b\x32\x1d.sdv.databroker.v1.Int32ArrayH\x00\x12\x34\n\x0bint64_array\x18\x18 \x01(\x0b\x32\x1d.sdv.databroker.v1.Int64ArrayH\x00\x12\x36\n\x0cuint32_array\x18\x19 \x01(\x0b\x32\x1e.sdv.databroker.v1.Uint32ArrayH\x00\x12\x36\n\x0cuint64_array\x18\x1a \x01(\x0b\x32\x1e.sdv.databroker.v1.Uint64ArrayH\x00\x12\x34\n\x0b\x66loat_array\x18\x1b \x01(\x0b\x32\x1d.sdv.databroker.v1.FloatArrayH\x00\x12\x36\n\x0c\x64ouble_array\x18\x1c \x01(\x0b\x32\x1e.sdv.databroker.v1.DoubleArrayH\x00\"m\n\x07\x46\x61ilure\x12\x11\n\rINVALID_VALUE\x10\x00\x12\x11\n\rNOT_AVAILABLE\x10\x01\x12\x15\n\x11UNKNOWN_DATAPOINT\x10\x02\x12\x11\n\rACCESS_DENIED\x10\x03\x12\x12\n\x0eINTERNAL_ERROR\x10\x04\x42\x07\n\x05value\"\xe0\x02\n\x08Metadata\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x30\n\nentry_type\x18\x02 \x01(\x0e\x32\x1c.sdv.databroker.v1.EntryType\x12\x0c\n\x04name\x18\x04 \x01(\t\x12.\n\tdata_type\x18\x05 \x01(\x0e\x32\x1b.sdv.databroker.v1.DataType\x12\x32\n\x0b\x63hange_type\x18\x06 \x01(\x0e\x32\x1d.sdv.databroker.v1.ChangeType\x12\x13\n\x0b\x64\x65scription\x18\x07 \x01(\t\x12+\n\x07\x61llowed\x18\n \x01(\x0b\x32\x1a.sdv.databroker.v1.Allowed\x12\x30\n\x03min\x18\x0b \x01(\x0b\x32#.sdv.databroker.v1.ValueRestriction\x12\x30\n\x03max\x18\x0c \x01(\x0b\x32#.sdv.databroker.v1.ValueRestriction\"\x9c\x03\n\x07\x41llowed\x12\x37\n\rstring_values\x18\x01 \x01(\x0b\x32\x1e.sdv.databroker.v1.StringArrayH\x00\x12\x35\n\x0cint32_values\x18\x03 \x01(\x0b\x32\x1d.sdv.databroker.v1.Int32ArrayH\x00\x12\x35\n\x0cint64_values\x18\x04 \x01(\x0b\x32\x1d.sdv.databroker.v1.Int64ArrayH\x00\x12\x37\n\ruint32_values\x18\x05 \x01(\x0b\x32\x1e.sdv.databroker.v1.Uint32ArrayH\x00\x12\x37\n\ruint64_values\x18\x06 \x01(\x0b\x32\x1e.sdv.databroker.v1.Uint64ArrayH\x00\x12\x35\n\x0c\x66loat_values\x18\x07 \x01(\x0b\x32\x1d.sdv.databroker.v1.FloatArrayH\x00\x12\x37\n\rdouble_values\x18\x08 \x01(\x0b\x32\x1e.sdv.databroker.v1.DoubleArrayH\x00\x42\x08\n\x06values\"\xac\x01\n\x10ValueRestriction\x12\x10\n\x06string\x18\x01 \x01(\tH\x00\x12\x0e\n\x04\x62ool\x18\x02 \x01(\x08H\x00\x12\x0f\n\x05int32\x18\x03 \x01(\x11H\x00\x12\x0f\n\x05int64\x18\x04 \x01(\x12H\x00\x12\x10\n\x06uint32\x18\x05 \x01(\rH\x00\x12\x10\n\x06uint64\x18\x06 \x01(\x04H\x00\x12\x0f\n\x05\x66loat\x18\x07 \x01(\x02H\x00\x12\x10\n\x06\x64ouble\x18\x08 \x01(\x01H\x00\x42\r\n\x0btyped_value*\xe0\x02\n\x08\x44\x61taType\x12\n\n\x06STRING\x10\x00\x12\x08\n\x04\x42OOL\x10\x01\x12\x08\n\x04INT8\x10\x02\x12\t\n\x05INT16\x10\x03\x12\t\n\x05INT32\x10\x04\x12\t\n\x05INT64\x10\x05\x12\t\n\x05UINT8\x10\x06\x12\n\n\x06UINT16\x10\x07\x12\n\n\x06UINT32\x10\x08\x12\n\n\x06UINT64\x10\t\x12\t\n\x05\x46LOAT\x10\n\x12\n\n\x06\x44OUBLE\x10\x0b\x12\x10\n\x0cSTRING_ARRAY\x10\x14\x12\x0e\n\nBOOL_ARRAY\x10\x15\x12\x0e\n\nINT8_ARRAY\x10\x16\x12\x0f\n\x0bINT16_ARRAY\x10\x17\x12\x0f\n\x0bINT32_ARRAY\x10\x18\x12\x0f\n\x0bINT64_ARRAY\x10\x19\x12\x0f\n\x0bUINT8_ARRAY\x10\x1a\x12\x10\n\x0cUINT16_ARRAY\x10\x1b\x12\x10\n\x0cUINT32_ARRAY\x10\x1c\x12\x10\n\x0cUINT64_ARRAY\x10\x1d\x12\x0f\n\x0b\x46LOAT_ARRAY\x10\x1e\x12\x10\n\x0c\x44OUBLE_ARRAY\x10\x1f*s\n\x0e\x44\x61tapointError\x12\x15\n\x11UNKNOWN_DATAPOINT\x10\x00\x12\x10\n\x0cINVALID_TYPE\x10\x01\x12\x11\n\rACCESS_DENIED\x10\x02\x12\x12\n\x0eINTERNAL_ERROR\x10\x03\x12\x11\n\rOUT_OF_BOUNDS\x10\x04*q\n\tEntryType\x12\x1a\n\x16\x45NTRY_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x45NTRY_TYPE_SENSOR\x10\x01\x12\x17\n\x13\x45NTRY_TYPE_ACTUATOR\x10\x02\x12\x18\n\x14\x45NTRY_TYPE_ATTRIBUTE\x10\x03*7\n\nChangeType\x12\n\n\x06STATIC\x10\x00\x12\r\n\tON_CHANGE\x10\x01\x12\x0e\n\nCONTINUOUS\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sdv.databroker.v1.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DATATYPE']._serialized_start=2143
  _globals['_DATATYPE']._serialized_end=2495
  _globals['_DATAPOINTERROR']._serialized_start=2497
  _globals['_DATAPOINTERROR']._serialized_end=2612
  _globals['_ENTRYTYPE']._serialized_start=2614
  _globals['_ENTRYTYPE']._serialized_end=2727
  _globals['_CHANGETYPE']._serialized_start=2729
  _globals['_CHANGETYPE']._serialized_end=2784
  _globals['_STRINGARRAY']._serialized_start=85
  _globals['_STRINGARRAY']._serialized_end=114
  _globals['_BOOLARRAY']._serialized_start=116
  _globals['_BOOLARRAY']._serialized_end=143
  _globals['_INT32ARRAY']._serialized_start=145
  _globals['_INT32ARRAY']._serialized_end=173
  _globals['_INT64ARRAY']._serialized_start=175
  _globals['_INT64ARRAY']._serialized_end=203
  _globals['_UINT32ARRAY']._serialized_start=205
  _globals['_UINT32ARRAY']._serialized_end=234
  _globals['_UINT64ARRAY']._serialized_start=236
  _globals['_UINT64ARRAY']._serialized_end=265
  _globals['_FLOATARRAY']._serialized_start=267
  _globals['_FLOATARRAY']._serialized_end=295
  _globals['_DOUBLEARRAY']._serialized_start=297
  _globals['_DOUBLEARRAY']._serialized_end=326
  _globals['_DATAPOINT']._serialized_start=329
  _globals['_DATAPOINT']._serialized_end=1195
  _globals['_DATAPOINT_FAILURE']._serialized_start=1077
  _globals['_DATAPOINT_FAILURE']._serialized_end=1186
  _globals['_METADATA']._serialized_start=1198
  _globals['_METADATA']._serialized_end=1550
  _globals['_ALLOWED']._serialized_start=1553
  _globals['_ALLOWED']._serialized_end=1965
  _globals['_VALUERESTRICTION']._serialized_start=1968
  _globals['_VALUERESTRICTION']._serialized_end=2140
# @@protoc_insertion_point(module_scope)
